@@ -31,6 +31,7 @@ using System.Data;
 using System.IO;
 
 using Hyena;
+using Hyena.Data;
 using Hyena.Data.Sqlite;
 using Hyena.Query;
 
@@ -42,7 +43,6 @@ using Banshee.Sources;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
-        using System.Diagnostics;
 // Disabling "is never used" warnings here because there are a lot
 // of properties/fields that are set via reflection at the database
 // layer - that is, they really are used, but the compiler doesn't
@@ -58,7 +58,7 @@ namespace Banshee.Collection.Database
         AbsoluteUri = 2
     }
 
-    public class DatabaseTrackInfo : TrackInfo, ICacheableItem
+    public class DatabaseTrackInfo : TrackInfo
     {
         private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (
             ServiceManager.DbConnection
@@ -98,21 +98,31 @@ namespace Banshee.Collection.Database
 
         public override bool TrackEqual (TrackInfo track)
         {
-            if (!(track is DatabaseTrackInfo)) {
-                return base.TrackEqual (track);
+            if (PrimarySource != null && PrimarySource.TrackEqualHandler != null) {
+                return PrimarySource.TrackEqualHandler (this, track);
             }
             
-            DatabaseTrackInfo db_track = (DatabaseTrackInfo)track;
-            return db_track.TrackId == TrackId && db_track.CacheModelId == CacheModelId && 
-                db_track.CacheEntryId == CacheEntryId;
+            DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
+            if (db_track == null) {
+                return base.TrackEqual (track);
+            }
+            return TrackEqual (this, db_track);
+        }
+        
+        public static bool TrackEqual (DatabaseTrackInfo a, DatabaseTrackInfo b)
+        {
+            return a != null && b != null && 
+                a.TrackId == b.TrackId && 
+                a.CacheModelId == b.CacheModelId && 
+                (int)a.CacheEntryId == (int)b.CacheEntryId;
         }
         
         public DatabaseArtistInfo Artist {
             get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
         }
-        
+
         public DatabaseAlbumInfo Album {
-            get { return DatabaseAlbumInfo.FindOrCreate (Artist, AlbumTitle); }
+            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle); }
         }
 
         private static bool notify_saved = true;
@@ -159,6 +169,11 @@ namespace Banshee.Collection.Database
         {
             Provider.Save (this);
         }
+
+        public void Refresh ()
+        {
+            ProviderRefresh ();
+        }
         
         protected virtual bool ProviderRefresh ()
         {
@@ -209,11 +224,16 @@ namespace Banshee.Collection.Database
                 if (value == null)
                     return;
 
+                // Overwrite the AlbumArtist if it was set to our old value
+                if (String.IsNullOrEmpty (AlbumArtist) || AlbumArtist == ArtistName) {
+                    AlbumArtist = value;
+                }
+
                 base.ArtistName = value;
                 artist_changed = artist_changed != null;
             }
         }
-        
+
         [VirtualDatabaseColumn ("Title", "CoreAlbums", "AlbumID", "AlbumID")]
         public override string AlbumTitle {
             get { return base.AlbumTitle; }
@@ -226,6 +246,19 @@ namespace Banshee.Collection.Database
                 album_changed = album_changed != null;
             }
         }
+
+        [VirtualDatabaseColumn ("ArtistName", "CoreAlbums", "AlbumID", "AlbumID")]
+        public override string AlbumArtist {
+            get { return base.AlbumArtist; }
+            set {
+                value = CleanseString (value, AlbumArtist);
+                if (value == null)
+                    return;
+
+                base.AlbumArtist = value;
+                album_changed = album_changed != null;
+            }
+        }
         
         private static string CleanseString (string input, string old_val)
         {
@@ -389,8 +422,13 @@ namespace Banshee.Collection.Database
         }
         
         [DatabaseColumn ("Rating")]
-        private int rating;
+        protected int rating;
         public override int Rating {
+            get { return rating; }
+            set { rating = value; }
+        }
+
+        public int SavedRating {
             get { return rating; }
             set {
                 if (rating != value) {
@@ -412,6 +450,13 @@ namespace Banshee.Collection.Database
             set { base.SkipCount = value; }
         }
         
+        private long external_id;
+        [DatabaseColumn ("ExternalID")]
+        public long ExternalId {
+            get { return external_id; }
+            set { external_id = value; }
+        }
+        
         [DatabaseColumn ("LastPlayedStamp")]
         public override DateTime LastPlayed {
             get { return base.LastPlayed; }
@@ -438,7 +483,7 @@ namespace Banshee.Collection.Database
         }
         
         [DatabaseColumn ("LastStreamError")]
-        private StreamPlaybackError playback_error;
+        protected StreamPlaybackError playback_error;
         public override StreamPlaybackError PlaybackError {
             get { return playback_error; }
             set {
@@ -451,29 +496,13 @@ namespace Banshee.Collection.Database
             }
         }
 
-#region Implement ICacheableItem
-
-        private long cache_entry_id;
-        public long CacheEntryId {
-            get { return cache_entry_id; }
-            set { cache_entry_id = value; }
-        }
-
-        private long cache_model_id;
-        public long CacheModelId {
-            get { return cache_model_id; }
-            set { cache_model_id = value; }
-        }
-
-#endregion
-
         private void UpdateUri ()
         {
             if (Uri == null && uri_type_set && UriField != null && PrimarySource != null) {
                 Uri = PrimarySource.UriAndTypeToSafeUri (UriType, UriField);
             }
         }
-        
+
         public void CopyToLibraryIfAppropriate (bool force_copy)
         {
             SafeUri old_uri = this.Uri;
@@ -482,11 +511,10 @@ namespace Banshee.Collection.Database
                 return;
             }
             
-            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocation);
-            //Console.WriteLine ("{0} is{1}in library.", old_uri.ToString (), in_library ? " " : " not ");
+            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocationWithSeparator);
 
             if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
-                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()).Substring (1));
+                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
@@ -497,7 +525,7 @@ namespace Banshee.Collection.Database
                     Banshee.IO.File.Copy (old_uri, new_uri, false);
                     Uri = new_uri;
                 } catch (Exception e) {
-                    Log.Error (String.Format("Exception copying into library: {0}", e), false);
+                    Log.ErrorFormat ("Exception copying into library: {0}", e);
                 }
             }
         }
@@ -514,20 +542,22 @@ namespace Banshee.Collection.Database
 
         public static int GetTrackIdForUri (string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, relative_path);
+            return GetTrackIdForUri (relative_path, relative_path, primary_sources);
+        }
+        
+        public static int GetTrackIdForUri (string uri, string relative_path, int [] primary_sources)
+        {
+            return ServiceManager.DbConnection.Query<int> (check_command, primary_sources, uri, relative_path);
         }
         
         public static bool ContainsUri (string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, relative_path) > 0;
+            return GetTrackIdForUri (relative_path, primary_sources) > 0;
         }
 
         public static bool ContainsUri (SafeUri uri, string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, uri.AbsoluteUri) > 0;
+            return GetTrackIdForUri (uri, relative_path, primary_sources) > 0;
         }
     }
 }
