@@ -55,9 +55,9 @@ namespace Banshee.Sources
             get { return when; }
         }
 
-        private QueryField changed_field;
-        public QueryField ChangedField {
-            get { return changed_field; }
+        private QueryField [] changed_fields;
+        public QueryField [] ChangedFields {
+            get { return changed_fields; }
         }
 
         public TrackEventArgs ()
@@ -65,14 +65,22 @@ namespace Banshee.Sources
             when = DateTime.Now;
         }
 
-        public TrackEventArgs (QueryField field) : this ()
+        public TrackEventArgs (params QueryField [] fields) : this ()
         {
-            this.changed_field = field;
+            changed_fields = fields;
         }
     }
 
+    public delegate bool TrackEqualHandler (DatabaseTrackInfo a, TrackInfo b);
+
     public abstract class PrimarySource : DatabaseSource, IDisposable
     {
+        private TrackEqualHandler track_equal_handler;
+        public TrackEqualHandler TrackEqualHandler {
+            get { return track_equal_handler; }
+            protected set { track_equal_handler = value; }
+        }
+    
         protected ErrorSource error_source;
         protected bool error_source_visible = false;
 
@@ -94,6 +102,8 @@ namespace Banshee.Sources
             DELETE FROM CoreTracks WHERE PrimarySourceId = ?
         ");
 
+        public readonly SchemaEntry<bool> ExpandedSchema;
+
         private int dbid;
         public int DbId {
             get {
@@ -116,6 +126,12 @@ namespace Banshee.Sources
             }
         }
 
+        private bool supports_playlists = true;
+        public bool SupportsPlaylists {
+            get { return supports_playlists; }
+            protected set { supports_playlists = value; }
+        }
+
         public ErrorSource ErrorSource {
             get {
                 if (error_source == null) {
@@ -127,6 +143,13 @@ namespace Banshee.Sources
             }
         }
 
+
+        private bool is_local = false;
+        public bool IsLocal {
+            get { return is_local; }
+            protected set { is_local = value; }
+        }
+
         public delegate void TrackEventHandler (Source sender, TrackEventArgs args);
 
         public event TrackEventHandler TracksAdded;
@@ -138,6 +161,18 @@ namespace Banshee.Sources
         {
             return (primary_sources.ContainsKey (id)) ? primary_sources[id] : null;
         }
+        
+        public virtual SafeUri UriToSafeUri (string uri_field)
+        {
+            if (String.IsNullOrEmpty (uri_field)) {
+                return null;
+            }
+            
+            return UriAndTypeToSafeUri (
+                uri_field[0] == System.IO.Path.DirectorySeparatorChar ? TrackUriType.AbsolutePath : TrackUriType.RelativePath,
+                uri_field
+            );
+        }
 
         public virtual SafeUri UriAndTypeToSafeUri (TrackUriType type, string uri_field)
         {
@@ -163,12 +198,29 @@ namespace Banshee.Sources
         protected PrimarySource (string generic_name, string name, string id, int order) : base (generic_name, name, id, order)
         {
             PrimarySourceInitialize ();
+
+            ExpandedSchema = new SchemaEntry<bool> (
+                String.Format ("sources.{0}", ConfigurationId), "expanded", true, "Is source expanded", "Is source expanded"
+            );
         }
 
         protected PrimarySource () : base ()
         {
+            ExpandedSchema = new SchemaEntry<bool> (
+                String.Format ("sources.{0}", ConfigurationId), "expanded", true, "Is source expanded", "Is source expanded"
+            );
         }
 
+        public override bool? AutoExpand {
+            get { return ExpandedSchema.Get (); }
+        }
+
+        public override bool Expanded {
+            get { return ExpandedSchema.Get (); }
+            set { ExpandedSchema.Set (value); }
+        }
+
+
         public virtual void Dispose ()
         {
             if (Application.ShuttingDown)
@@ -192,7 +244,7 @@ namespace Banshee.Sources
         private void PrimarySourceInitialize ()
         {
             // Scope the tracks to this primary source
-            track_model.Condition = String.Format ("CoreTracks.PrimarySourceID = {0}", DbId);
+            DatabaseTrackModel.AddCondition (String.Format ("CoreTracks.PrimarySourceID = {0}", DbId));
 
             primary_sources[DbId] = this;
             
@@ -240,17 +292,23 @@ namespace Banshee.Sources
             );
         }
 
+        public virtual void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
+        {
+            Log.WarningFormat ("CopyToTrack not implemented for source {0}", this);
+        }
+
         internal void NotifyTracksAdded ()
         {
             OnTracksAdded ();
         }
 
-        internal void NotifyTracksChanged (QueryField field)
+        internal void NotifyTracksChanged (params QueryField [] fields)
         {
-            OnTracksChanged (field);
+            OnTracksChanged (fields);
         }
 
-        internal void NotifyTracksChanged ()
+        // TODO replace this public method with a 'transaction'-like system
+        public void NotifyTracksChanged ()
         {
             OnTracksChanged ();
         }
@@ -262,12 +320,14 @@ namespace Banshee.Sources
 
         protected void OnErrorSourceUpdated (object o, EventArgs args)
         {
-            if (error_source.Count > 0 && !error_source_visible) {
-                AddChildSource (error_source);
-                error_source_visible = true;
-            } else if (error_source.Count <= 0 && error_source_visible) {
-                RemoveChildSource (error_source);
-                error_source_visible = false;
+            lock (error_source) {
+                if (error_source.Count > 0 && !error_source_visible) {
+                    error_source_visible = true;
+                    AddChildSource (error_source);
+                } else if (error_source.Count <= 0 && error_source_visible) {
+                    error_source_visible = false;
+                    RemoveChildSource (error_source);
+                }
             }
         }
 
@@ -314,14 +374,20 @@ namespace Banshee.Sources
             });
         }
 
-        protected override void OnTracksChanged (QueryField field)
+        protected override void OnTracksChanged (params QueryField [] fields)
         {
             ThreadAssist.SpawnFromMain (delegate {
-                Reload ();
+                if (NeedsReloadWhenFieldsChanged (fields)) {
+                    Reload ();
+                } else {
+                    InvalidateCaches ();
+                }
+
+                System.Threading.Thread.Sleep (150);
 
                 TrackEventHandler handler = TracksChanged;
                 if (handler != null) {
-                    handler (this, new TrackEventArgs (field));
+                    handler (this, new TrackEventArgs (fields));
                 }
             });
         }
@@ -428,7 +494,9 @@ namespace Banshee.Sources
 
         public override bool AcceptsInputFromSource (Source source)
         {
-            return base.AcceptsInputFromSource (source) && source.Parent != this;
+            return base.AcceptsInputFromSource (source) && source.Parent != this
+                && (source.Parent is PrimarySource || source is PrimarySource)
+                && !(source.Parent is Banshee.Library.LibrarySource);
         }
 
         public override bool AddSelectedTracks (Source source)
@@ -440,9 +508,18 @@ namespace Banshee.Sources
 
             // Store a snapshot of the current selection
             CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModelSelection (model);
-
             System.Threading.ThreadPool.QueueUserWorkItem (AddTrackList, cached_list);
-
+            return true;
+        }
+        
+        public override bool AddAllTracks (Source source)
+        {
+            if (!AcceptsInputFromSource (source) || source.Count == 0)
+                return false;
+            
+            DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
+            CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModel (model);
+            System.Threading.ThreadPool.QueueUserWorkItem (AddTrackList, cached_list);
             return true;
         }
 
@@ -470,6 +547,12 @@ namespace Banshee.Sources
             AddTrackJob.Total += (int) list.Count;
 
             foreach (DatabaseTrackInfo track in list) {
+                if (AddTrackJob.IsCancelRequested) {
+                    AddTrackJob.Finish ();
+                    IncrementAddedTracks ();
+                    break;
+                }
+                
                 if (track == null) {
                     IncrementAddedTracks ();
                     continue;
@@ -532,6 +615,7 @@ namespace Banshee.Sources
                             "Adding {0} of {1} to {2}"), "{0}", "{1}", Name), 
                             Properties.GetStringList ("Icon.Name"));
                         add_track_job.DelayShow = DelayAddJob;
+                        add_track_job.CanCancel = true;
                         add_track_job.Register ();
                     }
                 }
@@ -559,6 +643,8 @@ namespace Banshee.Sources
         {
             ServiceManager.DbConnection.Execute (prune_artists_albums_command);
             base.PruneArtistsAlbums ();
+            DatabaseAlbumInfo.Reset ();
+            DatabaseArtistInfo.Reset ();
         }
     }
 }
