@@ -42,12 +42,12 @@ namespace Banshee.Gui
 {
     public class GlobalActions : BansheeActionGroup
     {
-        public GlobalActions (InterfaceActionService actionService) : base ("Global")
+        public GlobalActions (InterfaceActionService actionService) : base (actionService, "Global")
         {
             Add (new ActionEntry [] {
-                // Music Menu
-                new ActionEntry ("MusicMenuAction", null, 
-                    Catalog.GetString ("_Music"), null, null, null),
+                // Media Menu
+                new ActionEntry ("MediaMenuAction", null, 
+                    Catalog.GetString ("_Media"), null, null, null),
 
                 new ActionEntry ("ImportAction", Stock.Open,
                     Catalog.GetString ("Import _Media..."), "<control>I",
@@ -91,13 +91,13 @@ namespace Banshee.Gui
                 new ActionEntry ("WikiGuideAction", Stock.Help,
                     Catalog.GetString ("Banshee _User Guide (Wiki)"), null,
                     Catalog.GetString ("Learn about how to use Banshee"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee-project.org/Guide");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/support/guide/");
                     }),
                     
                 new ActionEntry ("WikiSearchHelpAction", null,
                     Catalog.GetString ("Advanced Collection Searching"), null,
                     Catalog.GetString ("Learn advanced ways to search your media collection"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee-project.org/OnePointEx/Search");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/support/guide/searching/");
                     }),
                     
                 new ActionEntry ("WikiAction", null,
@@ -109,7 +109,7 @@ namespace Banshee.Gui
                 new ActionEntry ("WikiDeveloperAction", null,
                     Catalog.GetString ("_Get Involved"), null,
                     Catalog.GetString ("Become a contributor to Banshee"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee-project.org/Developers");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/contribute/");
                     }),
                  
                 new ActionEntry ("VersionInformationAction", null,
@@ -122,7 +122,7 @@ namespace Banshee.Gui
             this["ExtensionsAction"].Visible = false;
         }
             
-#region Music Menu Actions
+#region Media Menu Actions
 
         private void OnImport (object o, EventArgs args)
         {
@@ -154,78 +154,55 @@ namespace Banshee.Gui
                 radio_track.ParsingPlaylistEvent += delegate {
                     if (radio_track.PlaybackError != StreamPlaybackError.None) {
                         Log.Error (Catalog.GetString ("Error opening stream"), 
-                            Catalog.GetString ("Could not open stream or playlist"));
+                            Catalog.GetString ("Could not open stream or playlist"), true);
                         radio_track = null;
                     }
                 };
                 radio_track.Play ();
             } catch {
                 Log.Error (Catalog.GetString ("Error opening stream"), 
-                    Catalog.GetString("Problem parsing playlist"));
+                    Catalog.GetString("Problem parsing playlist"), true);
             }
         }
 
         private void OnImportPlaylist (object o, EventArgs args)
         {
             // Prompt user for location of the playlist.
-            Banshee.Gui.Dialogs.FileChooserDialog chooser = new Banshee.Gui.Dialogs.FileChooserDialog(
+            Banshee.Gui.Dialogs.FileChooserDialog chooser = new Banshee.Gui.Dialogs.FileChooserDialog (
                 Catalog.GetString("Import Playlist"),
                 PrimaryWindow,
                 FileChooserAction.Open
             );
                          
             chooser.DefaultResponse = ResponseType.Ok;
-            chooser.SelectMultiple = false;
+            chooser.SelectMultiple = true;
+            chooser.AddFilter (Hyena.Gui.GtkUtilities.GetFileFilter (Catalog.GetString ("Playlists"), PlaylistFileUtil.PlaylistExtensions));
+            try {
+                chooser.AddShortcutFolder (Paths.LibraryLocation);
+            } catch {}
 
-            chooser.AddButton(Stock.Cancel, ResponseType.Cancel);
-            chooser.AddButton(Catalog.GetString("Import"), ResponseType.Ok);
+            chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
+            // Translators: verb
+            chooser.AddButton (Catalog.GetString("Import"), ResponseType.Ok);
             
-            string playlist_uri = null;
             int response = chooser.Run();            
 
-            if(response == (int) ResponseType.Ok) {                    
-                playlist_uri = SafeUri.UriToFilename(chooser.Uri);              
-                chooser.Destroy(); 
+            string [] uris = null;
+            if (response == (int) ResponseType.Ok) {
+                uris = chooser.Uris;
+                chooser.Destroy();
             } else {
-                // User cancelled import.
-                chooser.Destroy();                 
-                return;
-            } 
-
-            // Read the contents of the playlist.
-            string[] uris = null;
-            try {                    
-                uris = PlaylistFileUtil.ImportPlaylist(playlist_uri);                    
-            } catch (Exception e) {
-                HigMessageDialog md = new HigMessageDialog(PrimaryWindow, 
-                    DialogFlags.DestroyWithParent, 
-                    MessageType.Error,  
-                    ButtonsType.Ok,
-                    Catalog.GetString("Unable to Import Playlist"),
-                    e.Message);
-
-                md.Run();
-                md.Destroy();
+                chooser.Destroy();
                 return;
             }
-
-            // Import the tracks specified in the playlist.
-            if (uris != null) {
-                ImportPlaylistWorker worker = new ImportPlaylistWorker(playlist_uri, uris);
-                worker.Import ();
-            } else {
-                HigMessageDialog md = new HigMessageDialog(PrimaryWindow, 
-                    DialogFlags.DestroyWithParent, 
-                    MessageType.Error,  
-                    ButtonsType.Ok,
-                    Catalog.GetString("Unable to Import Playlist"),
-                    Catalog.GetString("Banshee was unable to find any valid tracks to import.  Please check the playlist and try again.")
-                );
-
-                md.Run();
-                md.Destroy();
+            
+            if (uris == null || uris.Length == 0) {
                 return;
             }
+            
+            foreach (string uri in uris) {
+                PlaylistFileUtil.ImportPlaylistToLibrary (uri);
+            }
         }
         
         private void OnQuit (object o, EventArgs args)
@@ -239,9 +216,12 @@ namespace Banshee.Gui
 
         private void OnPreferences (object o, EventArgs args)
         {
-            Dialogs.PreferencesDialog dialog = new Dialogs.PreferencesDialog ();
-            dialog.Run ();
-            dialog.Destroy ();
+            try {
+                Banshee.Preferences.Gui.PreferenceDialog dialog = new Banshee.Preferences.Gui.PreferenceDialog ();
+                dialog.Run ();
+                dialog.Destroy ();
+            } catch (ApplicationException) {
+            }
         }
 
         private void OnExtensions (object o, EventArgs args)
