@@ -4,7 +4,7 @@
 // Author:
 //   Gabriel Burt <gburt@novell.com>
 //
-// Copyright (C) 2007 Novell, Inc.
+// Copyright (C) 2007-2008 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -86,12 +86,12 @@ namespace Banshee.Gui
                     OnAddToNewPlaylist),
 
                 new ActionEntry ("RemoveTracksAction", Stock.Remove,
-                    Catalog.GetString("_Remove"), "Delete",
-                    Catalog.GetString("Remove selected track(s) from this source"), OnRemoveTracks),
+                    Catalog.GetString ("_Remove"), "Delete",
+                    Catalog.GetString ("Remove selected track(s) from this source"), OnRemoveTracks),
 
                 new ActionEntry ("RemoveTracksFromLibraryAction", null,
-                    Catalog.GetString("Remove From _Library"), "",
-                    Catalog.GetString("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
+                    Catalog.GetString ("Remove From _Library"), "",
+                    Catalog.GetString ("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
 
                 new ActionEntry ("DeleteTracksFromDriveAction", null,
                     Catalog.GetString ("_Delete From Drive"), null,
@@ -111,10 +111,6 @@ namespace Banshee.Gui
                 new ActionEntry ("SearchForSameArtistAction", null,
                     Catalog.GetString ("By Matching A_rtist"), null,
                     Catalog.GetString ("Search all songs of this artist"), OnSearchForSameArtist),
-
-                //new ActionEntry ("JumpToPlayingTrackAction", null,
-                //    Catalog.GetString ("_Jump to playing song"), "<control>J",
-                //    null, OnJumpToPlayingTrack),
             });
 
             Actions.UIManager.ActionsChanged += HandleActionsChanged;
@@ -200,14 +196,16 @@ namespace Banshee.Gui
         {
             Source source = ServiceManager.SourceManager.ActiveSource;
             bool in_database = source is DatabaseSource;
-            PrimarySource primary_source = source as PrimarySource ?? source.Parent as PrimarySource;
+            PrimarySource primary_source = (source as PrimarySource) ?? (source.Parent as PrimarySource);
             
             Hyena.Collections.Selection selection = (source is ITrackModelSource) ? (source as ITrackModelSource).TrackModel.Selection : null;
 
             if (selection != null) {
+                Sensitive = Visible = true;
                 bool has_selection = selection.Count > 0;
-                foreach (string action in require_selection_actions)
+                foreach (string action in require_selection_actions) {
                     this[action].Sensitive = has_selection;
+                }
 
                 bool has_single_selection = selection.Count == 1;
 
@@ -225,11 +223,12 @@ namespace Banshee.Gui
                     UpdateAction ("DeleteTracksFromDriveAction", is_track_source && track_source.CanDeleteTracks, has_selection, source);
                     UpdateAction ("RemoveTracksFromLibraryAction", source.Parent is LibrarySource, has_selection, null);
                     
-                    UpdateAction ("TrackPropertiesAction", in_database, has_selection, null);
+                    UpdateAction ("TrackPropertiesAction", in_database, has_selection, source);
                     UpdateAction ("RateTracksAction", in_database, has_selection, null);
-
                     UpdateAction ("AddToPlaylistAction", in_database && primary_source != null && primary_source.SupportsPlaylists, has_selection, null);
                 }
+            } else {
+                Sensitive = Visible = false;
             }
         }
 
@@ -273,10 +272,16 @@ namespace Banshee.Gui
         private void OnTrackProperties (object o, EventArgs args)
         {
             if (current_source != null) {
-                TrackEditor propEdit = new TrackEditor (current_source.TrackModel.SelectedItems);
-                propEdit.Saved += delegate {
-                    //ui.playlistView.QueueDraw();
-                };
+                Source source = current_source as Source;
+                InvokeHandler handler = source != null 
+                    ? source.GetInheritedProperty<InvokeHandler> ("TrackPropertiesActionHandler") 
+                    : null;
+                
+                if (handler != null) {
+                    handler ();
+                } else {
+                    new TrackEditor (current_source.TrackModel.SelectedItems);
+                }
             }
         }
 
@@ -407,10 +412,6 @@ namespace Banshee.Gui
             }
         }
 
-        /*private void OnJumpToPlayingTrack (object o, EventArgs args)
-        {
-        }*/
-
 #endregion
 
         private static bool ConfirmRemove (ITrackModelSource source, bool delete, int selCount)
