@@ -146,6 +146,8 @@ namespace Banshee.Dap.Mtp
             try {
                 List<Track> files = mtp_device.GetAllTracks (delegate (ulong current, ulong total, IntPtr data) {
                     //user_event.Progress = (double)current / total;
+                    // Translators: {0} is the name of the MTP audio device (eg Gabe's Zen Player), {1} is the
+                    // current track being loaded, and {2} is the total # of tracks that will be loaded.
                     SetStatus (String.Format (Catalog.GetString ("Loading {0} - {1} of {2}"), Name, current, total), false);
                     return 0;
                 });
@@ -177,6 +179,18 @@ namespace Banshee.Dap.Mtp
             //new LibraryImportManager (true).QueueSource (BaseDirectory);
         }
 
+        public override void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
+        {
+            if (track_map.ContainsKey (track.TrackId)) {
+                track_map[track.TrackId].Download (uri.LocalPath, delegate (ulong current, ulong total, IntPtr data) {
+                    job.DetailedProgress = (double) current / total;
+                    return 0;
+                });
+            } else {
+                throw new Exception ("Error copying track from MTP device");
+            }
+        }
+
         public override bool CanRename {
             get { return !(IsAdding || IsDeleting); }
         }
