@@ -52,7 +52,7 @@ namespace Banshee.Dap
         {
             base.Initialize ();
 
-            Order = 175;
+            Order = 410;
             Properties.SetString ("UnmapSourceActionIconName", "media-eject");
             Properties.SetString ("GtkActionPath", "/RemovableSourceContextMenu");
             AfterInitialized ();
@@ -87,6 +87,10 @@ namespace Banshee.Dap
         public virtual bool CanImport {
             get { return true; }
         }
+        
+        int IImportSource.SortOrder {
+            get { return 20; }
+        }
 
 #region IUnmapableSource Implementation
 
@@ -115,6 +119,11 @@ namespace Banshee.Dap
             return true;
         }
 
+        public override bool AcceptsInputFromSource (Source source)
+        {
+            return (source is DatabaseSource) && this != source.Parent;
+        }
+
         private bool syncing = false;
         public bool CanUnmap {
             get { return !syncing; }
