@@ -48,33 +48,30 @@ namespace Banshee.Daap
         private DAAP.Service service;
         private DAAP.Client client;
         private DAAP.Database database;
+        private bool connected = false;
         
         public DAAP.Database Database {
             get { return database; }
         }
-        
-        private Dictionary <int, int> daap_track_map;
-        public Dictionary <int, int> TrackMap {
-            get { return daap_track_map; }
-        }
-        
+
         private bool is_activating;
-        private int playlistid;
         
         public DaapSource (DAAP.Service service) : base (Catalog.GetString ("Music Share"), service.Name, 
                                                     (service.Address.ToString () + service.Port).Replace (":", "").Replace (".", ""), 300)
         {
             this.service = service;
-            daap_track_map = new Dictionary <int, int> ();
             Properties.SetString ("UnmapSourceActionLabel", Catalog.GetString ("Disconnect"));
             Properties.SetString ("UnmapSourceActionIconName", "gtk-disconnect");
             
+            SupportsPlaylists = false;
+            SavedCount = 0;
             UpdateIcon ();
             
             AfterInitialized ();
-            playlistid = this.DbId;
         }
         
+        
+        
         private void UpdateIcon ()
         {
             if (service != null && !service.IsProtected) {
@@ -84,19 +81,29 @@ namespace Banshee.Daap
             }
         }
         
+        public override void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
+        {
+            if (track.PrimarySource == this && track.Uri.Scheme.StartsWith ("http")) {
+                foreach (double percent in database.DownloadTrack ((int)track.ExternalId, track.MimeType, uri.AbsolutePath)) {
+                    job.DetailedProgress = percent;
+                }
+            }
+        }
+
+        
         public override void Activate ()
         {
             if (client != null || is_activating) {
-            	return;
+                return;
             }
             
+            ClearErrorView ();
+            
             is_activating = true;
             base.Activate ();
             
             SetStatus (String.Format (Catalog.GetString ("Connecting to {0}"), service.Name), false);
             
-            Console.WriteLine ("Connecting to {0}:{1}", service.Address, service.Port);
-            
             ThreadAssist.Spawn (delegate {
                 try {
                     client = new DAAP.Client (service);
@@ -108,28 +115,30 @@ namespace Banshee.Daap
                         ThreadAssist.ProxyToMain (PromptLogin);
                     }
                 } catch(Exception e) {
-                    /*ThreadAssist.ProxyToMain(delegate {
-                        DaapErrorView error_view = new DaapErrorView(this, DaapErrorType.BrokenAuthentication);
-                        while(box.Children.Length > 0) {
-                            box.Remove(box.Children[0]);
-                        }
-                        box.PackStart(error_view, true, true, 0);
-                        error_view.Show();
-                    });*/
-                    
-                    string details = String.Format ("Couldn't connect to service {0} on {1}:{2} - {3}",
-                                                      service.Name,
-                                                      service.Address,
-                                                      service.Port, e.ToString ().Replace ("&", "&amp;")
-                                                    .Replace ("<", "&lt;").Replace (">", "&gt;"));
-                    Log.Warning ("Failed to connect", details, true);
-                    HideStatus ();
+                    ShowErrorView (DaapErrorType.BrokenAuthentication);
+                    Hyena.Log.Exception (e);
                 }
                
                 is_activating = false;
             });
         }
         
+        private void ShowErrorView (DaapErrorType error_type)
+        {
+            PurgeTracks ();
+            Reload ();
+            client = null;
+            DaapErrorView error_view = new DaapErrorView (this, error_type);
+            error_view.Show ();
+            Properties.Set<Banshee.Sources.Gui.ISourceContents> ("Nereid.SourceContents", error_view);
+            HideStatus ();
+        }
+        
+        private void ClearErrorView ()
+        {
+            Properties.Remove ("Nereid.SourceContents");
+        }
+        
         internal bool Disconnect (bool logout)
         {
             // Stop currently playing track if its from us.
@@ -140,15 +149,13 @@ namespace Banshee.Daap
                         ServiceManager.PlayerEngine.Close ();
                     }
                 }
-            } catch { }
+            } catch {}
+            
+            connected = false;
             
             // Remove tracks associated with this source, since we don't want
             // them after we unmap - we'll refetch.
-            if (Count > 0) {
-                RemoveTrackRange ((DatabaseTrackListModel)TrackModel, new Hyena.Collections.RangeCollection.Range (0, Count));
-            }
-            
-            daap_track_map.Clear ();
+            PurgeTracks ();
             
             if (client != null) {
                 if (logout) {
@@ -161,12 +168,22 @@ namespace Banshee.Daap
             }
             
             if (database != null) {
+                try {
+                    DaapService.ProxyServer.UnregisterDatabase (database);
+                } catch {}
                 database.TrackAdded -= OnDatabaseTrackAdded;
                 database.TrackRemoved -= OnDatabaseTrackRemoved;
                 database = null;
             }
-
-            ClearChildSources();
+            
+            List<Source> children = new List<Source> (Children);
+            foreach (Source child in children) {
+                if (child is Banshee.Sources.IUnmapableSource) {
+                    (child as Banshee.Sources.IUnmapableSource).Unmap ();
+                }
+            }
+            
+            ClearChildSources ();
             
             return true;
         }
@@ -177,16 +194,16 @@ namespace Banshee.Daap
             base.Dispose ();
         }
         
-        private void PromptLogin (object o, EventArgs args)
+        private void PromptLogin ()
         {
-            SetStatus (String.Format (Catalog.GetString ("Logging in to {0}"), Name), false);
+            SetStatus (Catalog.GetString ("Logging in to {0}."), false);
             
             DaapLoginDialog dialog = new DaapLoginDialog (client.Name, 
             client.AuthenticationMethod == DAAP.AuthenticationMethod.UserAndPassword);
             if (dialog.Run () == (int) Gtk.ResponseType.Ok) {
                 AuthenticatedLogin (dialog.Username, dialog.Password);
             } else {
-                Unmap ();
+                ShowErrorView (DaapErrorType.UserDisconnect);
             }
 
             dialog.Destroy ();
@@ -198,47 +215,76 @@ namespace Banshee.Daap
                 try {
                     client.Login (username, password);
                 } catch (DAAP.AuthenticationException) {
-                    ThreadAssist.ProxyToMain (PromptLogin);
+                    ThreadAssist.ProxyToMain (delegate {
+                        ShowErrorView (DaapErrorType.InvalidAuthentication);
+                    });
                 }
             });
         }
         
         private void OnClientUpdated (object o, EventArgs args)
         {
-            if (database == null && client.Databases.Count > 0) {
-                database = client.Databases[0];
-                database.TrackAdded += OnDatabaseTrackAdded;
-                database.TrackRemoved += OnDatabaseTrackRemoved;
-                
-                foreach (DAAP.Track track in database.Tracks) {
-                    DaapTrackInfo daaptrack = new DaapTrackInfo (track, this);
-                    daaptrack.Save ();
-                    Log.Debug ("Track ID", daaptrack.TrackId.ToString ());
+            try {
+                if (database == null && client.Databases.Count > 0) {
+                    database = client.Databases[0];
+                    DaapService.ProxyServer.RegisterDatabase (database);
+                    database.TrackAdded += OnDatabaseTrackAdded;
+                    database.TrackRemoved += OnDatabaseTrackRemoved;
                     
-                    daap_track_map.Add (track.Id, daaptrack.TrackId);
+                    SetStatus (String.Format (Catalog.GetPluralString (
+                        "Loading {0} track.", "Loading {0} tracks.", database.Tracks.Count),
+                        database.Tracks.Count), false
+                    );
+                    
+                    // Notify (eg reload the source before sync is done) at most 5 times
+                    int notify_every = Math.Max (250, (database.Tracks.Count / 4));
+                    notify_every -= notify_every % 250;
+                    
+                    int count = 0;
+                    DaapTrackInfo daap_track = null;
+                    foreach (DAAP.Track track in database.Tracks) {
+                        daap_track = new DaapTrackInfo (track, this);
+                        
+                        // Only notify once in a while because otherwise the source Reloading slows things way down
+                        daap_track.Save (++count % notify_every == 0);
+                    }
+                    
+                    // Save the last track once more to trigger the NotifyTrackAdded
+                    if (daap_track != null) {
+                        daap_track.Save ();
+                    }
+                    
+                    SetStatus (Catalog.GetString ("Loading playlists"), false);
+                    AddPlaylistSources ();
+                    connected = true;
+                    Reload ();
+                    HideStatus ();
                 }
                 
-                AddPlaylistSources ();
+                Name = client.Name;
                 
-                Reload ();
-                
-                ThreadAssist.ProxyToMain(delegate {
+                UpdateIcon ();
+                OnUpdated ();
+            } catch (Exception e) {
+                Hyena.Log.Exception ("Caught exception while loading daap share", e);
+                ThreadAssist.ProxyToMain (delegate {
                     HideStatus ();
+                    ShowErrorView (DaapErrorType.UserDisconnect);
                 });
             }
-            
-            Name = client.Name;
-            
-            UpdateIcon ();
-            OnUpdated ();
         }
         
         private void AddPlaylistSources ()
         {
             foreach (DAAP.Playlist pl in database.Playlists) {
-                DaapPlaylistSource source = new DaapPlaylistSource (pl, playlistid, this);
-                AddChildSource (source);
-                playlistid ++;
+                DaapPlaylistSource source = new DaapPlaylistSource (pl, this);
+                ThreadAssist.ProxyToMain (delegate {
+                    if (source.Count == 0) {
+                        source.Unmap ();
+                    } else {
+                        AddChildSource (source);
+                    }
+                });
             }
         }
         
@@ -246,20 +292,11 @@ namespace Banshee.Daap
         {
             DaapTrackInfo track = new DaapTrackInfo (args.Track, this);
             track.Save ();
-            
-            if (!daap_track_map.ContainsKey (args.Track.Id)) {
-                daap_track_map.Add (args.Track.Id, track.TrackId);
-            }
         }
         
         public void OnDatabaseTrackRemoved (object o, DAAP.TrackArgs args)
         {
-            if (daap_track_map.ContainsKey (args.Track.Id)) {
-                //DaapTrackInfo track = daap_track_map [args.Track.Id];
-                //RemoveTrack (track);
-                //RemoveTrack (
-                // TODO
-            }
+            //RemoveTrack (
         }
         
         public override bool CanRemoveTracks {
@@ -276,19 +313,15 @@ namespace Banshee.Daap
 
         public bool Unmap ()
         {
-            // TODO: Maybe keep track of where we came from, or pick the next source up on the list?
-            ServiceManager.SourceManager.SetActiveSource (ServiceManager.SourceManager.MusicLibrary);
-            
             // Disconnect and clear out our tracks and such.
             Disconnect (true);
-            
-            Reload ();
+            ShowErrorView (DaapErrorType.UserDisconnect);
             
             return true;
         }
         
         public bool CanUnmap {
-            get { return true; }
+            get { return connected; }
         }
         
         public bool ConfirmBeforeUnmap {
@@ -297,16 +330,15 @@ namespace Banshee.Daap
         
         public void Import ()
         {
-            Log.Debug ("Starting import...");
-            DateTime start = DateTime.Now;
-            ServiceManager.SourceManager.MusicLibrary.AddAllTracks (this);
-            DateTime finish = DateTime.Now;
-            TimeSpan time = finish - start;
-            Log.DebugFormat ("Import completed. Took {0} seconds.", time.TotalSeconds);
+            ServiceManager.SourceManager.MusicLibrary.MergeSourceInput (this, SourceMergeType.All);
         }
         
         public bool CanImport {
-            get { return true; }
+            get { return connected; }
+        }
+        
+        int IImportSource.SortOrder {
+            get { return 30; }
         }
         
         public string [] IconNames {
