@@ -157,8 +157,7 @@ namespace Banshee.NotificationArea
                 }
             }
 
-            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
-               PlayerEvent.Iterate |
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent,
                PlayerEvent.StartOfStream |
                PlayerEvent.EndOfStream |
                PlayerEvent.TrackInfoUpdated |
@@ -319,14 +318,9 @@ namespace Banshee.NotificationArea
         private void OnPlayerEvent (PlayerEventArgs args) 
         {
             switch (args.Event) {
-                case PlayerEvent.Iterate:
-                    if (current_track != ServiceManager.PlayerEngine.CurrentTrack) {
-                        current_track = ServiceManager.PlayerEngine.CurrentTrack;
-                        ShowTrackNotification ();
-                    }
-                    break;
                 case PlayerEvent.StartOfStream:
                 case PlayerEvent.TrackInfoUpdated:
+                    current_track = ServiceManager.PlayerEngine.CurrentTrack;
                     ToggleRatingMenuSensitive ();
                     ShowTrackNotification ();
                     break;
@@ -341,9 +335,16 @@ namespace Banshee.NotificationArea
         
         private void OnRatingChanged (object o, EventArgs args)
         {
-            if (ServiceManager.PlayerEngine.CurrentTrack != null) {
-                ServiceManager.PlayerEngine.CurrentTrack.Rating = rating_menu_item.Value;
-                ServiceManager.PlayerEngine.CurrentTrack.Save (); //true, Banshee.Query.BansheeQuery.RatingField);
+            TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
+            if (track != null) {
+                DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
+                if (db_track != null) {
+                    db_track.SavedRating = rating_menu_item.Value;
+                } else {
+                    track.Rating = rating_menu_item.Value;
+                    track.Save ();
+                }
+
                 ServiceManager.PlayerEngine.TrackInfoUpdated ();
             }
         }
