@@ -51,6 +51,7 @@ namespace Banshee.PlayQueue
     {
         private static string special_playlist_name = "Play Queue";//typeof (PlayQueueSource).ToString ();
 
+        private ITrackModelSource prior_playback_source;
         private DatabaseTrackInfo playing_track;
         private bool actions_loaded = false;
 
@@ -58,15 +59,15 @@ namespace Banshee.PlayQueue
             get { return false; }
         }
         
-        public PlayQueueSource () : base (Catalog.GetString ("Play Queue"), null, 5)
+        public PlayQueueSource () : base (Catalog.GetString ("Play Queue"), null, 20)
         {
             BindToDatabase ();
             
-            Order = 10;
+            Order = 20;
             Properties.SetString ("Icon.Name", "source-playlist");
             Properties.SetString ("RemoveTracksActionLabel", Catalog.GetString ("Remove From Play Queue"));
             
-            ((DatabaseTrackListModel)TrackModel).ForcedSortQuery = "CorePlaylistEntries.EntryID ASC";
+            ((DatabaseTrackListModel)TrackModel).ForcedSortQuery = "CorePlaylistEntries.ViewOrder ASC, CorePlaylistEntries.EntryID ASC";
             
             ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent);
             ServiceManager.PlaybackController.Transition += OnCanonicalPlaybackControllerTransition;
@@ -114,8 +115,10 @@ namespace Banshee.PlayQueue
             ServiceManager.SourceManager.VideoLibrary.TracksDeleted += HandleTracksDeleted;
             
             TrackModel.Reloaded += delegate {
-                if (this == ServiceManager.PlaybackController.Source && Count == 0) {
-                    ServiceManager.PlaybackController.Source = (ITrackModelSource)ServiceManager.SourceManager.DefaultSource;
+                if (Count == 0) {
+                    if (this == ServiceManager.PlaybackController.Source || this == ServiceManager.PlaybackController.NextSource) {
+                        ServiceManager.PlaybackController.NextSource = ServiceManager.PlaybackController.Source = PriorSource;
+                    }
                 }
             };
             
@@ -127,6 +130,7 @@ namespace Banshee.PlayQueue
         private void SetAsPlaybackSourceUnlessPlaying ()
         {
             if (Count > 0) {
+                PriorSource = ServiceManager.PlaybackController.Source;
                 ServiceManager.PlaybackController.NextSource = this;
             }
         }
@@ -174,6 +178,7 @@ namespace Banshee.PlayQueue
         private void OnCanonicalPlaybackControllerTransition (object o, EventArgs args)
         {
             if (Count > 0) {
+                PriorSource = ServiceManager.PlaybackController.Source;
                 ServiceManager.PlaybackController.Source = this;
             }
         }
@@ -238,7 +243,7 @@ namespace Banshee.PlayQueue
             RemovePlayingTrack ();
             
             if (Count == 0) {
-                ServiceManager.PlaybackController.Source = (ITrackModelSource)ServiceManager.SourceManager.DefaultSource;
+                ServiceManager.PlaybackController.Source = PriorSource;
                 ServiceManager.PlaybackController.Next (restart);
                 return;
             }
@@ -258,6 +263,21 @@ namespace Banshee.PlayQueue
             }
         }
         
+        private ITrackModelSource PriorSource {
+            get {
+                if (prior_playback_source == null || prior_playback_source == this) {
+                    return (ITrackModelSource)ServiceManager.SourceManager.DefaultSource;
+                }
+                return prior_playback_source;
+            }
+            set {
+                if (value == null || value == this) {
+                    return;
+                }
+                prior_playback_source = value;
+            }
+        }
+        
         public override bool CanRename {
             get { return false; }
         }
